from confluent_kafka import Producer
from confluent_kafka.schema_registry import SchemaRegistryClient
from confluent_kafka.schema_registry.protobuf import ProtobufSerializer
from confluent_kafka.serialization import MessageField, SerializationContext
from google.protobuf.message import Message
from .vertica_datacontract_pb2 import Issue

class KafkaProducer:
    def __init__(
        self,
        bootstrap_servers: str,
        schema_registry_url: str,
        client_id: str = "python-producer",
    ):
        self.producer = Producer(
            {
                "bootstrap.servers": bootstrap_servers,
                "client.id": client_id,
            }
        )

        self.schema_registry_client = SchemaRegistryClient(
            conf={"url": schema_registry_url}
        )
        self.serializers_cache = {}

    def _get_serializer(self, proto_message_type):
        if proto_message_type not in self.serializers_cache:
            self.serializers_cache[proto_message_type] = ProtobufSerializer(
                msg_type=proto_message_type,
                schema_registry_client=self.schema_registry_client,
            )
        return self.serializers_cache[proto_message_type]

    def send_to_kafka(self, proto_message: Message, topic: str, key=None):
        serializer = self._get_serializer(proto_message.__class__)

        value = serializer(
            message=proto_message,
            ctx=SerializationContext(topic=topic, field=MessageField.VALUE),
        )
        

        self.producer.produce(
            topic=topic,
            key=key,
            value=value,
        )

    def flush(self, timeout=5.0):
        self.producer.flush(timeout=timeout)
    
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.flush()


# Предположим, у нас есть сгенерированный protobuf класс
# from my_protobuf_pb2 import MyMessage

# Создаем продюсер

with KafkaProducer(
    bootstrap_servers="localhost:9092",
    schema_registry_url="http://localhost:8081",
    client_id="my-app-producer"
) as producer:
    
    # Создаем и отправляем сообщение
    message = Issue()
    message.row_id = 123
    message.id = "test_message_1"
    
    producer.send_to_kafka(
        proto_message=message,
        topic="vertica_datacontract",
        # key="key-123"  # опционально
    )
    # Можно отправлять несколько сообщений
    # for i in range(5):
    #     msg = Issue()
    #     msg.row_id = i
    #     msg.id = f"message_{i}"
        
    #     producer.send_to_kafka(
    #         proto_message=msg,
    #         topic="vertica_datacontract",
    #         # key=f"key-{i}"
    #     )
# producer = KafkaProducer(
#     bootstrap_servers="localhost:9092",
#     schema_registry_url="http://localhost:8081"
# )

# # Создаем protobuf сообщение
# message = Issue()
# message.row_id = 123
# message.id = "test message"
# # message.timestamp = 1700000000
# print(message)
# # Отправляем сообщение
# producer.send_to_kafka(
#     proto_message=message,
#     topic="my-topic",
#     key="message-key-123"  # опционально
# )

# Можно отправлять несколько сообщений
# for i in range(10):
#     msg = MyMessage()
#     msg.id = i
#     msg.name = f"message_{i}"
#     producer.send_to_kafka(msg, "my-topic", f"key_{i}")

# Финальная отправка всех сообщений
# producer.flush()

# uv run python -m src.tests.test_send_to_kafka


import logging
import sys
from pathlib import Path

import click

from helpers import ModuleBuilder, WheelBuilder


# Add the current directory to the import path
current_dir = Path(__file__).parent.absolute()
sys.path.insert(0, str(object=current_dir))

# https://datacontract.com/examples/orders-latest/datacontract.yaml




log: logging.Logger = logging.getLogger(name=__name__)
log.setLevel(level=logging.DEBUG)



@click.group()
@click.pass_context
def cli(
    ctx,
):
    ctx.ensure_object(dict)



@cli.command()
@click.option("--filename")
@click.option("--subject-name")
def publish_schema_registry(
    filename: str,
    subject_name: str,
):
    """
    uv run --env-file .env python -m src publish-schema-registry --filename "et_admin_datacontract" --subject-name et_admin_datacontract

    uv run datacontract-helper publish-schema-registry --filename et_admin_datacontract --subject-name et_admin_datacontract
    
    """
    ModuleBuilder().publish_schema_registry(
        filename=filename, subject_name=subject_name
    )


@cli.command()
@click.option("--filename", default="vertica_datacontract")
@click.option(
    "--subject-name",
    default="vertica_datacontract",
)
@click.option("--version", default="latest")
@click.option("--compatibility-type", default="FULL")
def validate_schema_registry(
    filename: str = "vertica_datacontract",
    subject_name: str = "vertica_datacontract",
    version: str = "latest",
    compatibility_type: str = "FULL",
):
    """

    uv run --env-file .env python -m src validate-schema-registry --filename "vertica_datacontract" --subject-name vertica_datacontract --compatibility-type FULL
    
    uv run datacontract-helper validate-schema-registry --filename "et_admin_datacontract" --subject-name et_admin_datacontract --compatibility-type FULL
    """
    ModuleBuilder().validate_custom(filename=filename)

    ModuleBuilder().validate_schema_registry(
        subject_name=subject_name,
        version=version,
        filename=filename,
        compatibility_type=compatibility_type,
    )        


@cli.command()
@click.option("--filename", required=False, type=str, help="Название файла")
def validate_custom(filename: str):
    """
    uv run --env-file .env python -m src validate-custom --filename vertica_datacontract

    uv run datacontract-helper validate-custom --filename vertica_datacontract

    """

    ModuleBuilder().validate_custom(filename=filename)


@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def create_yaml_from_sql(filename: str):
    """
    нужен ddl.sql

    uv run --env-file .env python -m src create-yaml-from-sql --filename vertica_datacontract

    uv run datacontract-helper create-yaml-from-sql --filename vertica_datacontract
    """
    ModuleBuilder().create_yaml_from_sql(filename=filename)



@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def create_proto_from_yaml(filename: str):
    """
    нужен your-datacontract.yaml

    uv run --env-file .env python -m src create-proto-from-yaml --filename vertica_datacontract

    uv run datacontract-helper create-proto-from-yaml --filename et_admin_datacontract

    """
    ModuleBuilder().create_proto_from_yaml(filename=filename)


@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def generate_python_code_from_proto(filename: str):
    """
    uv run --env-file .env python -m src generate-python-code-from-proto --filename vertica_datacontract

    uv run datacontract-helper generate-python-code-from-proto --filename et_admin_datacontract

    """
    ModuleBuilder().generate_python_code_from_proto(filename=filename)


@cli.command()
@click.option("--wheel-version", required=True)
@click.option("--proto-file-name", default="vertica_datacontract_pb2")
@click.option("--filename", default="vertica_datacontract")
def create_wheel(
    wheel_version: str,
    proto_file_name: str = "vertica_datacontract_pb2",
    filename: str = "vertica_datacontract",
):
    """
    uv run --env-file .env python -m src create-wheel --proto-file-name vertica_datacontract_pb2 --wheel-version 0.1.9 --filename vertica_datacontract

    uv run datacontract-helper create-wheel --proto-file-name et_admin_datacontract_pb2 --wheel-version 0.1.5 --filename et_admin_datacontract

    """

    WheelBuilder().build_wheel(
        proto_file_name=proto_file_name,
        filename=filename,
        version=wheel_version,
    )


@cli.command()
@click.option("--filepath", required=True, type=str)
@click.option("--nexusurl", required=True, type=str)
@click.option("--username", required=True, type=str)
@click.option("--password", required=True, type=str)
def publish_package(
    filepath: str, nexusurl: str, username: str, password: str
):
    """

    uv run python -m src publish-package --filepath "vertica_datacontract-0.1.9-py3-none-any.whl" --nexusurl "https://nexus.k8s-analytics.ostrovok.in/repository/datacontract_pypi/" --username n.shokurov --password test_pass

    uv run datacontract-helper create-wheel --proto-file-name et_admin_datacontract_pb2 --wheel-version 0.1.4 --filename et_admin_datacontract


    """
    ModuleBuilder().publish_package(
        nexus_pass=password,
        nexus_repo=nexusurl,
        nexus_user=username,
        wheel_file=filepath
    )



import logging
import os
import shutil
import subprocess
import tempfile
from datacontract.data_contract import DataContract
import click

# NEXUS_URL: str = os.environ["NEXUS_URL"]
# NEXUS_USERNAME: str = os.environ["NEXUS_USERNAME"]
# NEXUS_PASSWORD: str = os.environ["NEXUS_PASSWORD"]


log = logging.getLogger(name="").getChild(suffix=__name__)


SEND_TO_KAFKA_TXT: str = """

from confluent_kafka import Producer
from confluent_kafka.schema_registry import SchemaRegistryClient
from confluent_kafka.schema_registry.protobuf import ProtobufSerializer
from confluent_kafka.serialization import MessageField, SerializationContext
from google.protobuf.message import Message

class KafkaProducer:
    def __init__(
        self,
        username: str,
        password: str,
        bootstrap_servers: str = broker,
        schema_registry_url: str = schema_registry,
        client_id: str = "datacontract-python-producer",
    ):
        self.producer = Producer(
            {
                "bootstrap.servers": bootstrap_servers,
                'security.protocol': 'SASL_PLAINTEXT',
                'sasl.mechanisms': 'PLAIN',
                'sasl.username': username,
                'sasl.password': password,
                "client.id": client_id,
            }
        )

        self.schema_registry_client = SchemaRegistryClient(
            conf={"url": schema_registry_url}
        )
        self.serializers_cache = {}

    def _get_serializer(self, proto_message_type):
        if proto_message_type not in self.serializers_cache:
            self.serializers_cache[proto_message_type] = ProtobufSerializer(
                msg_type=proto_message_type,
                schema_registry_client=self.schema_registry_client,
            )
        return self.serializers_cache[proto_message_type]

    def send_to_kafka(
        self,
        proto_message: Message,
        topic: str = topic,
        key=None):
        serializer = self._get_serializer(proto_message.__class__)

        value = serializer(
            message=proto_message,
            ctx=SerializationContext(topic=topic, field=MessageField.VALUE),
        )
        

        self.producer.produce(
            topic=topic,
            key=key,
            value=value,
        )

    def flush(self, timeout=5.0):
        self.producer.flush(timeout=timeout)
    
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.flush()



"""


class WheelBuilder:

    def __init__(self): ...

    def build_wheel(
        self,
        proto_file_name: str = "vertica_datacontract_pb2",
        filename: str = "vertica_datacontract",
        version: str = "0.1.8",
    ):
        module_folder = filename

        # VERSION: str = "0.1.8"

        # MODULE_NAME: str = "vertica-datacontract-tool"
        # MODULE_NAME: str = "vertica_datacontract_tool"

        # у нас должен быть файл proto.py:
        # PROTO_FILE_NAME: str = "vertica_datacontract_pb2"

        # MODULE_FOLDER: str = "vertica_datacontract"

        # pyproject_toml_content: str = """
        # [build-system]
        # requires = ["setuptools", "wheel"]
        # build-backend = "setuptools.build_meta"
        # """

        setup_file_content: str = f"""
from setuptools import setup, find_packages

setup(
    name="{module_folder}",
    version="{version}",# my_package.__version__,
    # author=my_package.__author__,
    packages=find_packages(),
    install_requires=[
        "protobuf>=3.20.0",
        "build>=1.3.0",
        "confluent-kafka[all]>=2.11.1",
        "pip>=25.2",
        ],

)
"""

        with tempfile.TemporaryDirectory() as tmpdirname:
            setup_file: str = os.path.join(tmpdirname, "setup.py")

            with open(setup_file, "w") as f:
                f.write(setup_file_content)

            module_folder_path: str = os.path.join(tmpdirname, module_folder)
            os.makedirs(module_folder_path)  # , exist_ok=True)

            init_file: str = os.path.join(module_folder_path, "__init__.py")

            with open(f"{proto_file_name}.py", "r") as proto1:
                with open(init_file, "w") as f:
                    f.write(proto1.read())
                    f.write("\n\n")

                    f.write("\n".join(self.get_tags(filename=filename)))
                    f.write("\n\n")
                    f.write(
                        "\n".join(
                            f'{key} = "{value}"'
                            for key, value in self.get_urls(filename=filename).items()
                        )
                    )
                    f.write("\n\n")
                    f.write(SEND_TO_KAFKA_TXT)

            with open(init_file, "r") as f:
                click.echo("its __init__.py")
                click.echo(f.read())
            with open(setup_file, "r") as f:
                click.echo("its setup.py")
                click.echo(f.read())

            subprocess.run(
                args=["python", "setup.py", "bdist_wheel"],
                check=True,
                cwd=tmpdirname,  # указать working directory
            )

            dist_folder: str = os.path.join(tmpdirname, "dist")
            wheels: list[str] = [
                os.path.join(dist_folder, filepath)
                for filepath in os.listdir(dist_folder)
            ]
            click.echo({"wheels": wheels})

            for wheel in wheels:
                subprocess.run(
                    args=["python", "-m", "pip", "install", wheel],
                    check=True,
                    cwd=tmpdirname,  # указать working directory
                )

            real_dir = os.getcwd()
            for wheel in wheels:
                click.echo(f"try copy2 {wheel}")
                shutil.copy2(src=wheel, dst=real_dir)

    def get_tags(self, filename: str) -> list[str]:
        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        return data_contract.get_data_contract_specification().tags

    def get_urls(self, filename: str) -> dict:
        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()
        # run = data_contract.test()
        # if not run.has_passed():
        #     raise Exception("Data quality validation failed.")
        # kafka://
        
        return {key: value.replace("kafka://", "") if key == "broker" else value for key, value in data_contract.get_data_contract_specification().links.items()}
        # data_contract.get_data_contract_specification().links["schema_registry"]
        # data_contract.get_data_contract_specification().links["broker"]
